package Source::Repository::Mass::SkipPkg::ImportSkipMassRebuild;

use strict;
use warnings;
use Carp;
use File::Basename;
use RPM::Header;
use RPM::Vercmp;

# due to Source::Repository::Mass::version_comparator
use Source::Repository::Mass;
use Source::Package::Comparators::Raw;
use Source::Repository::Mass::SkipPkg;
our @ISA=qw/Source::Repository::Mass::SkipPkg/;
push @Source::Repository::Mass::PkgFilter::PKG_SKIP_CLASSES, __PACKAGE__;

our $verbose=1;
our $arg_enable_filter=0;
our $arg_skip_rebuilds=1;
our $arg_rebuilds_depth=0; # number of rebuilds to skip

our @LONGOPT = (
    "changelog-mass-rebuild-on"	=> 	sub {$arg_enable_filter=1},
    "changelog-mass-rebuild-off"	=> sub {$arg_enable_filter=0},
    "changelog-mass-rebuild-select"	=> sub {$arg_enable_filter=1,$arg_skip_rebuilds=0},
    "changelog-mass-rebuild-skip"	=> sub {$arg_enable_filter=1,$arg_skip_rebuilds=1},
    "changelog-mass-rebuild-depth=s"	=> \$arg_rebuilds_depth,
    );

sub print_longopt {
    print "  `Ignore changelog Mass Rebuild' options:
	--changelog-mass-rebuild-skip  enable skippkg of mass rebuilds
	--changelog-mass-rebuild-depth <num> number of mass rebuilds to skip
";
}

sub pkgfilter {
    my ($self,$pkgpair)=@_;
    return 0 unless $arg_enable_filter;
    my $filter_call=$self->pkgfilter_if_skip_rebuilds($pkgpair);
    if ($arg_skip_rebuilds) {
	print STDERR "note: skipped Mass_Rebuild ",$pkgpair->originname,"\n" if $filter_call and $verbose;
	return $filter_call;
    } else {
	print STDERR "note: selected Mass_Rebuild ",$pkgpair->originname,"\n" if not $filter_call and $verbose;
	return ! $filter_call;
    }
}


sub pkgfilter_if_skip_rebuilds {
    my ($self,$pkgpair)=@_;
    my $origin=$pkgpair->origin;
    my $dest=$pkgpair->dest;
    return 0 unless $dest;
    unless ($origin) {
	my $originname=$pkgpair->originname;
	my $destname=$pkgpair->destname;
	print STDERR "Oops! no origin found for $originname ($destname). Skipping\n";
	return 1;
    }
    #return 0 if $origin->version ne $dest->version;
    return 0 if &$Source::Repository::Mass::version_comparator($origin,$dest)!=0
	# hack; we should ignore epoch if version is the same; epoch can be dropped during import,
	# an example is aspell-pt_BR epoch 50 dropped in aspell.pl 
	# TODO: better use version part of version comparator 
	and $origin->version ne $dest->version;
    my $altoriginrel=&__get_origin_release_from_alt($dest->release);
    return 0 if ! defined($altoriginrel);
    my $depth=&__calculate_rebuild_depth($origin, $altoriginrel);
    my $originrel=&Source::Package::Comparators::Raw::import_trim_release($origin->release);
    print STDERR "DEBUG: skip Mass_Rebuild ",$pkgpair->originname,": altoriginrel=$altoriginrel originrel=$originrel depth=$depth\n" if $verbose>2;
    return 0 if $depth==0;
    return 0 if &__number($originrel)-&__number($altoriginrel) > $depth;
    return 0 if $arg_rebuilds_depth and $depth > $arg_rebuilds_depth;
    return 1;
}

#------------------------------------------
#    private utilities
#------------------------------------------

sub __number {
    my $rel=shift;
    $rel=$1 if $rel=~m/^(\d+(?:\.\d+)?)/;
    return $rel;
}

sub __calculate_rebuild_depth {
    my ($origin,$altoriginrel)=@_;
    my $originfile=$origin->file;
    die "Oops! file $originfile not found" if ! -e $originfile;
    my $rh=RPM::Header->new($originfile);
    die "Oops! file $originfile: RPM::Header failed" if ! $rh;
    # from last to first
    my $changelognameref=$rh->{CHANGELOGNAME};
    my $changelogtextref=$rh->{CHANGELOGTEXT};
    die "Oops! bad changelog" unless $changelognameref or $changelogtextref;
    my $curversion=$origin->version();
    my $depth=0;
    for (my $i=0; $i<@$changelognameref; $i++) {
	my ($ver,$rel)=&__get_epochversion_release($changelognameref->[$i]);
	# ignore epoch
	$ver=~s/^\d+://;
	return $depth if $ver ne $curversion;
	return $depth if RPM::Vercmp::evrcmp($rel,$altoriginrel)<=0;
	my $is_rebuild=&__is_rebuild($changelognameref->[$i],$changelogtextref->[$i]);
	return $depth if !$is_rebuild;
	$depth++;
    }
    return $depth;
}

sub __get_epochversion_release {
    my ($changelogname)=@_;
    # 'Fedora Release Engineering <releng@fedoraproject.org> - 0.95-13'
    if ($changelogname=~/-\s+([^-]+)-([^-]+)$/) {
	return ($1,$2);
    # umeabot <umeabot> 1.0.4-7.mga7
    } elsif ($changelogname=~/\>\s+([^-]+)-([^-]+)$/) {
	return ($1,$2);
    } else {
	warn "Oops! bad changelogname: $changelogname";
	return (-1,0);
    }
}

sub __is_rebuild {
    my ($changelogname, $changelogtext)=@_;
    # umeabot <umeabot> 1.0.4-7.mga7 # Mageia 7 Mass Rebuild
    return 1 if $changelogname=~/umeabot \<umeabot\>/ and $changelogtext=~m!Mageia [\d\.]+ Mass Rebuild!;
    # 'Fedora Release Engineering <releng@fedoraproject.org> - 0.95-13'
    return 1 if $changelogname=~/Fedora Release Engineering/ and $changelogtext=~m!Rebuilt for https://fedoraproject.org/wiki/!;
    #- Rebuild for readline 8.0
    return 1 if $changelogtext=~m!^- Rebuil[dt] for \S+ \S+\s*$!;
    return 0;
}

sub __get_origin_release_from_alt {
    my $altrel=shift;
    my $originrel;
    if ($altrel=~/^(?:[^_]+)_(.*?)(?:jpp\d+)?$/) {
	$originrel=$1;
    }
    return $originrel;
}

1;

__END__
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_28_Mass_Rebuild',
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Binutils_Mass_Rebuild',
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Mass_Rebuild',
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_26_Mass_Rebuild',
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_24_Mass_Rebuild',
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_23_Mass_Rebuild',
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_21_22_Mass_Rebuild',
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_21_Mass_Rebuild',
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_19_Mass_Rebuild',
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_18_Mass_Rebuild',
'- Rebuilt for https://fedoraproject.org/wiki/Fedora_17_Mass_Rebuild',
* Sun Sep 23 2018 umeabot <umeabot> 1.0.4-7.mga7
  (not released yet)
+ Revision: 1297284
- Mageia 7 Mass Rebuild
- Rebuild for readline 8.0
- Rebuilt for Python 3.8
